@@ -53,7 +53,10 @@ class Application(object):
                 self.cfg.set(k.lower(), v)
 
         # Load up the config file if its found.
-        if args.config and os.path.exists(args.config):
+        if args.config:
+            if not os.path.exists(args.config):
+                raise RuntimeError("%r doesn't exist" % args.config)
+
             cfg = {
                 "__builtins__": __builtins__,
                 "__name__": "__config__",
@@ -116,14 +119,15 @@ class Application(object):
 
         if self.cfg.spew:
             debug.spew()
+
         if self.cfg.daemon:
-            util.daemonize()
+            util.daemonize(self.cfg.enable_stdio_inheritance)
 
         # set python paths
         if self.cfg.pythonpath and self.cfg.pythonpath is not None:
             paths = self.cfg.pythonpath.split(",")
             for path in paths:
-                pythonpath = os.path.abspath(self.cfg.pythonpath)
+                pythonpath = os.path.abspath(path)
                 if pythonpath not in sys.path:
                     sys.path.insert(0, pythonpath)
 
