@@ -19,7 +19,9 @@ class WSGIApplication(Application):
         self.cfg.set("default_proc_name", args[0])
         self.app_uri = args[0]
 
-        sys.path.insert(0, os.getcwd())
+        cwd = util.getcwd()
+
+        sys.path.insert(0, cwd)
 
     def load(self):
         return util.import_app(self.app_uri)
@@ -27,8 +29,12 @@ class WSGIApplication(Application):
 
 def run():
     """\
-    The ``gunicorn`` command line runner for launcing Gunicorn with
+    The ``gunicorn`` command line runner for launching Gunicorn with
     generic WSGI applications.
     """
     from gunicorn.app.wsgiapp import WSGIApplication
     WSGIApplication("%(prog)s [OPTIONS] APP_MODULE").run()
+
+
+if __name__ == '__main__':
+    run()
