@@ -17,6 +17,33 @@ import threading
 from gunicorn import util
 from gunicorn.six import string_types
 
+
+# syslog facility codes
+SYSLOG_FACILITIES = {
+        "auth":     4,
+        "authpriv": 10,
+        "cron":     9,
+        "daemon":   3,
+        "ftp":      11,
+        "kern":     0,
+        "lpr":      6,
+        "mail":     2,
+        "news":     7,
+        "security": 4,  #  DEPRECATED
+        "syslog":   5,
+        "user":     1,
+        "uucp":     8,
+        "local0":   16,
+        "local1":   17,
+        "local2":   18,
+        "local3":   19,
+        "local4":   20,
+        "local5":   21,
+        "local6":   22,
+        "local7":   23
+        }
+
+
 CONFIG_DEFAULTS = dict(
         version=1,
         disable_existing_loggers=False,
@@ -54,56 +81,6 @@ def loggers():
     return [logging.getLogger(name) for name in existing]
 
 
-class LazyWriter(object):
-
-    """
-    File-like object that opens a file lazily when it is first written
-    to.
-    """
-
-    def __init__(self, filename, mode='w'):
-        self.filename = filename
-        self.fileobj = None
-        self.lock = threading.Lock()
-        self.mode = mode
-
-    def open(self):
-        if self.fileobj is None:
-            self.lock.acquire()
-            try:
-                if self.fileobj is None:
-                    self.fileobj = open(self.filename, self.mode)
-            finally:
-                self.lock.release()
-        return self.fileobj
-
-    def close(self):
-        if self.fileobj:
-            self.lock.acquire()
-            try:
-                if self.fileobj:
-                    self.fileobj.close()
-                    self.fileobj = None
-            finally:
-                self.lock.release()
-
-    def write(self, text):
-        fileobj = self.open()
-        fileobj.write(text)
-        fileobj.flush()
-
-    def writelines(self, text):
-        fileobj = self.open()
-        fileobj.writelines(text)
-        fileobj.flush()
-
-    def flush(self):
-        self.open().flush()
-
-    def isatty(self):
-        return bool(self.fileobj and self.fileobj.isatty())
-
-
 class SafeAtoms(dict):
 
     def __init__(self, atoms):
@@ -175,12 +152,16 @@ class Logger(object):
     access_fmt = "%(message)s"
     syslog_fmt = "[%(process)d] %(message)s"
 
+    atoms_wrapper_class = SafeAtoms
+
     def __init__(self, cfg):
         self.error_log = logging.getLogger("gunicorn.error")
         self.access_log = logging.getLogger("gunicorn.access")
         self.error_handlers = []
         self.access_handlers = []
         self.cfg = cfg
+        self.logfile = None
+        self.lock = threading.Lock()
         self.setup(cfg)
 
     def setup(self, cfg):
@@ -192,7 +173,12 @@ class Logger(object):
             if cfg.errorlog != "-":
                 # if an error log file is set redirect stdout & stderr to
                 # this log file.
-                sys.stdout = sys.stderr = LazyWriter(cfg.errorlog, 'a')
+                for stream in sys.stdout, sys.stderr:
+                    stream.flush()
+
+                self.logfile = open(cfg.errorlog, 'a+')
+                os.dup2(self.logfile.fileno(), sys.stdout.fileno())
+                os.dup2(self.logfile.fileno(), sys.stderr.fileno())
 
             # set gunicorn.error handler
             self._set_handler(self.error_log, cfg.errorlog,
@@ -237,30 +223,25 @@ class Logger(object):
             lvl = self.LOG_LEVELS.get(lvl.lower(), logging.INFO)
         self.error_log.log(lvl, msg, *args, **kwargs)
 
-    def access(self, resp, req, environ, request_time):
-        """ Seee http://httpd.apache.org/docs/2.0/logs.html#combined
-        for format details
+    def atoms(self, resp, req, environ, request_time):
+        """ Gets atoms for log formating.
         """
-
-        if not self.cfg.accesslog and not self.cfg.logconfig:
-            return
-
         status = resp.status.split(None, 1)[0]
         atoms = {
-                'h': environ.get('REMOTE_ADDR', '-'),
-                'l': '-',
-                'u': '-',  # would be cool to get username from basic auth header
-                't': self.now(),
-                'r': "%s %s %s" % (environ['REQUEST_METHOD'],
-                    environ['RAW_URI'], environ["SERVER_PROTOCOL"]),
-                's': status,
-                'b': resp.response_length and str(resp.response_length) or '-',
-                'f': environ.get('HTTP_REFERER', '-'),
-                'a': environ.get('HTTP_USER_AGENT', '-'),
-                'T': str(request_time.seconds),
-                'D': str(request_time.microseconds),
-                'p': "<%s>" % os.getpid()
-                }
+            'h': environ.get('REMOTE_ADDR', '-'),
+            'l': '-',
+            'u': '-',  # would be cool to get username from basic auth header
+            't': self.now(),
+            'r': "%s %s %s" % (environ['REQUEST_METHOD'],
+                environ['RAW_URI'], environ["SERVER_PROTOCOL"]),
+            's': status,
+            'b': resp.response_length and str(resp.response_length) or '-',
+            'f': environ.get('HTTP_REFERER', '-'),
+            'a': environ.get('HTTP_USER_AGENT', '-'),
+            'T': str(request_time.seconds),
+            'D': str(request_time.microseconds),
+            'p': "<%s>" % os.getpid()
+        }
 
         # add request headers
         if hasattr(req, 'headers'):
@@ -273,10 +254,21 @@ class Logger(object):
         # add response headers
         atoms.update(dict([("{%s}o" % k.lower(), v) for k, v in resp.headers]))
 
+        return atoms
+
+    def access(self, resp, req, environ, request_time):
+        """ See http://httpd.apache.org/docs/2.0/logs.html#combined
+        for format details
+        """
+
+        if not self.cfg.accesslog and not self.cfg.logconfig:
+            return
+
         # wrap atoms:
         # - make sure atoms will be test case insensitively
         # - if atom doesn't exist replace it by '-'
-        safe_atoms = SafeAtoms(atoms)
+        safe_atoms = self.atoms_wrapper_class(self.atoms(resp, req, environ,
+            request_time))
 
         try:
             self.access_log.info(self.cfg.access_log_format % safe_atoms)
@@ -292,9 +284,19 @@ class Logger(object):
 
     def reopen_files(self):
         if self.cfg.errorlog != "-":
-            # Close stderr & stdout if they are redirected to error log file
-            sys.stderr.close()
-            sys.stdout.close()
+            # if an error log file is set redirect stdout & stderr to
+            # this log file.
+            for stream in sys.stdout, sys.stderr:
+                stream.flush()
+
+            with self.lock:
+                if self.logfile is not None:
+                    self.logfile.close()
+
+                self.logfile = open(self.cfg.errorlog, 'a+')
+                os.dup2(self.logfile.fileno(), sys.stdout.fileno())
+                os.dup2(self.logfile.fileno(), sys.stderr.fileno())
+
         for log in loggers():
             for handler in log.handlers:
                 if isinstance(handler, logging.FileHandler):
@@ -347,13 +349,22 @@ class Logger(object):
             prefix = cfg.syslog_prefix
 
         prefix = "gunicorn.%s" % prefix
+
+        # set format
         fmt = logging.Formatter(r"%s: %s" % (prefix, fmt))
 
+        # syslog facility
+        try:
+            facility = SYSLOG_FACILITIES[cfg.syslog_facility.lower()]
+        except KeyError:
+            raise RuntimeError("unknown facility name")
+
         # parse syslog address
         socktype, addr = parse_syslog_address(cfg.syslog_addr)
 
         # finally setup the syslog handler
-        h = logging.handlers.SysLogHandler(address=addr, socktype=socktype)
+        h = logging.handlers.SysLogHandler(address=addr, facility=facility,
+                socktype=socktype)
         h.setFormatter(fmt)
         h._gunicorn = True
         log.addHandler(h)
