@@ -4,16 +4,6 @@
 # See the NOTICE for more information.
 
 
-try:
-    import ctypes
-except MemoryError:
-    # selinux execmem denial
-    # https://bugzilla.redhat.com/show_bug.cgi?id=488396
-    ctypes = None
-except ImportError:
-    # Python on Solaris compiled with Sun Studio doesn't have ctypes
-    ctypes = None
-
 import fcntl
 import os
 import pkg_resources
@@ -23,17 +13,16 @@ import socket
 import sys
 import textwrap
 import time
+import traceback
 import inspect
 import errno
 import warnings
 
+from gunicorn.errors import AppImportError
 from gunicorn.six import text_type, string_types
 
 MAXFD = 1024
-if (hasattr(os, "devnull")):
-    REDIRECT_TO = os.devnull
-else:
-    REDIRECT_TO = "/dev/null"
+REDIRECT_TO = getattr(os, 'devnull', '/dev/null')
 
 timeout_default = object()
 
@@ -118,7 +107,12 @@ def load_class(uri, default="sync", section="gunicorn.workers"):
             dist = entry_str
             name = default
 
-        return pkg_resources.load_entry_point(dist, section, name)
+        try:
+            return pkg_resources.load_entry_point(dist, section, name)
+        except:
+            exc = traceback.format_exc()
+            raise RuntimeError("class uri %r invalid or not found: \n\n[%s]" % (uri,
+                exc))
     else:
         components = uri.split('.')
         if len(components) == 1:
@@ -128,11 +122,19 @@ def load_class(uri, default="sync", section="gunicorn.workers"):
 
                 return pkg_resources.load_entry_point("gunicorn",
                             section, uri)
-            except ImportError as e:
-                raise RuntimeError("class uri invalid or not found: " +
-                        "[%s]" % str(e))
+            except:
+                exc = traceback.format_exc()
+                raise RuntimeError("class uri %r invalid or not found: \n\n[%s]" % (uri,
+                    exc))
+
         klass = components.pop(-1)
-        mod = __import__('.'.join(components))
+        try:
+            mod = __import__('.'.join(components))
+        except:
+            exc = traceback.format_exc()
+            raise RuntimeError("class uri %r invalid or not found: \n\n[%s]" % (uri,
+                exc))
+
         for comp in components[1:]:
             mod = getattr(mod, comp)
         return getattr(mod, klass)
@@ -141,26 +143,17 @@ def load_class(uri, default="sync", section="gunicorn.workers"):
 def set_owner_process(uid, gid):
     """ set user and group of workers processes """
     if gid:
-        try:
-            os.setgid(gid)
-        except OverflowError:
-            if not ctypes:
-                raise
-            # versions of python < 2.6.2 don't manage unsigned int for
-            # groups like on osx or fedora
-            os.setgid(-ctypes.c_int(-gid).value)
-
+        # versions of python < 2.6.2 don't manage unsigned int for
+        # groups like on osx or fedora
+        gid = abs(gid) & 0x7FFFFFFF
+        os.setgid(gid)
     if uid:
         os.setuid(uid)
 
 
 def chown(path, uid, gid):
-    try:
-        os.chown(path, uid, gid)
-    except OverflowError:
-        if not ctypes:
-            raise
-        os.chown(path, uid, -ctypes.c_int(-gid).value)
+    gid = abs(gid) & 0x7FFFFFFF  # see note above.
+    os.chown(path, uid, gid)
 
 
 if sys.platform.startswith("win"):
@@ -217,16 +210,21 @@ def is_ipv6(addr):
         socket.inet_pton(socket.AF_INET6, addr)
     except socket.error:  # not a valid address
         return False
+    except ValueError: # ipv6 not supported on this platform
+        return False
     return True
 
 
 def parse_address(netloc, default_port=8000):
-    if netloc.startswith("unix:"):
-        return netloc.split("unix:")[1]
-
     if netloc.startswith("unix://"):
         return netloc.split("unix://")[1]
 
+    if netloc.startswith("unix:/"):
+        return netloc.split("unix:/")[1]
+
+    if netloc.startswith("unix:"):
+        return netloc.split("unix:")[1]
+
     if netloc.startswith("tcp://"):
         netloc = netloc.split("tcp://")[1]
 
@@ -364,14 +362,34 @@ def import_app(module):
             raise
 
     mod = sys.modules[module]
-    app = eval(obj, mod.__dict__)
+
+    try:
+        app = eval(obj, mod.__dict__)
+    except NameError:
+        raise AppImportError("Failed to find application: %r" % module)
+
     if app is None:
-        raise ImportError("Failed to find application object: %r" % obj)
+        raise AppImportError("Failed to find application object: %r" % obj)
+
     if not callable(app):
-        raise TypeError("Application object must be callable.")
+        raise AppImportError("Application object must be callable.")
     return app
 
 
+def getcwd():
+    # get current path, try to use PWD env first
+    try:
+        a = os.stat(os.environ['PWD'])
+        b = os.stat(os.getcwd())
+        if a.st_ino == b.st_ino and a.st_dev == b.st_dev:
+            cwd = os.environ['PWD']
+        else:
+            cwd = os.getcwd()
+    except:
+        cwd = os.getcwd()
+    return cwd
+
+
 def http_date(timestamp=None):
     """Return the current date and time formatted for a message header."""
     if timestamp is None:
@@ -388,7 +406,7 @@ def is_hoppish(header):
     return header.lower().strip() in hop_headers
 
 
-def daemonize():
+def daemonize(enable_stdio_inheritance=False):
     """\
     Standard daemonization of a process.
     http://www.svbug.com/documentation/comp.unix.programmer-FAQ/faq_2.html#SEC16
@@ -402,12 +420,73 @@ def daemonize():
             os._exit(0)
 
         os.umask(0)
-        maxfd = get_maxfd()
-        closerange(0, maxfd)
 
-        os.open(REDIRECT_TO, os.O_RDWR)
-        os.dup2(0, 1)
-        os.dup2(0, 2)
+        # In both the following any file descriptors above stdin
+        # stdout and stderr are left untouched. The inheritence
+        # option simply allows one to have output go to a file
+        # specified by way of shell redirection when not wanting
+        # to use --error-log option.
+
+        if not enable_stdio_inheritance:
+            # Remap all of stdin, stdout and stderr on to
+            # /dev/null. The expectation is that users have
+            # specified the --error-log option.
+
+            closerange(0, 3)
+
+            fd_null = os.open(REDIRECT_TO, os.O_RDWR)
+
+            if fd_null != 0:
+                os.dup2(fd_null, 0)
+
+            os.dup2(fd_null, 1)
+            os.dup2(fd_null, 2)
+
+        else:
+            fd_null = os.open(REDIRECT_TO, os.O_RDWR)
+
+            # Always redirect stdin to /dev/null as we would
+            # never expect to need to read interactive input.
+
+            if fd_null != 0:
+                os.close(0)
+                os.dup2(fd_null, 0)
+
+            # If stdout and stderr are still connected to
+            # their original file descriptors we check to see
+            # if they are associated with terminal devices.
+            # When they are we map them to /dev/null so that
+            # are still detached from any controlling terminal
+            # properly. If not we preserve them as they are.
+            #
+            # If stdin and stdout were not hooked up to the
+            # original file descriptors, then all bets are
+            # off and all we can really do is leave them as
+            # they were.
+            #
+            # This will allow 'gunicorn ... > output.log 2>&1'
+            # to work with stdout/stderr going to the file
+            # as expected.
+            #
+            # Note that if using --error-log option, the log
+            # file specified through shell redirection will
+            # only be used up until the log file specified
+            # by the option takes over. As it replaces stdout
+            # and stderr at the file descriptor level, then
+            # anything using stdout or stderr, including having
+            # cached a reference to them, will still work.
+
+            def redirect(stream, fd_expect):
+                try:
+                    fd = stream.fileno()
+                    if fd == fd_expect and stream.isatty():
+                        os.close(fd)
+                        os.dup2(fd_null, fd)
+                except AttributeError:
+                    pass
+
+            redirect(sys.stdout, 1)
+            redirect(sys.stderr, 2)
 
 
 def seed():
