@@ -3,11 +3,10 @@
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
-from __future__ import with_statement
-
 from datetime import datetime
 import errno
 import socket
+import ssl
 
 import gunicorn.http as http
 import gunicorn.http.wsgi as wsgi
@@ -17,6 +16,7 @@ from gunicorn import six
 
 ALREADY_HANDLED = object()
 
+
 class AsyncWorker(base.Worker):
 
     def __init__(self, *args, **kwargs):
@@ -26,14 +26,14 @@ class AsyncWorker(base.Worker):
     def timeout_ctx(self):
         raise NotImplementedError()
 
-    def handle(self, client, addr):
+    def handle(self, listener, client, addr):
         req = None
         try:
             parser = http.RequestParser(self.cfg, client)
             try:
                 if not self.cfg.keepalive:
                     req = six.next(parser)
-                    self.handle_request(req, client, addr)
+                    self.handle_request(listener, req, client, addr)
                 else:
                     # keepalive loop
                     while True:
@@ -42,15 +42,24 @@ class AsyncWorker(base.Worker):
                             req = six.next(parser)
                         if not req:
                             break
-                        self.handle_request(req, client, addr)
+                        self.handle_request(listener, req, client, addr)
             except http.errors.NoMoreData as e:
                 self.log.debug("Ignored premature client disconnection. %s", e)
             except StopIteration as e:
                 self.log.debug("Closing connection. %s", e)
+            except ssl.SSLError:
+                raise  # pass to next try-except level
             except socket.error:
-                raise # pass to next try-except level
+                raise  # pass to next try-except level
             except Exception as e:
                 self.handle_error(req, client, addr, e)
+        except ssl.SSLError as e:
+            if e.args[0] == ssl.SSL_ERROR_EOF:
+                self.log.debug("ssl connection closed")
+                client.close()
+            else:
+                self.log.debug("Error processing SSL request.")
+                self.handle_error(req, client, addr, e)
         except socket.error as e:
             if e.args[0] not in (errno.EPIPE, errno.ECONNRESET):
                 self.log.exception("Socket error processing request.")
@@ -64,11 +73,14 @@ class AsyncWorker(base.Worker):
         finally:
             util.close(client)
 
-    def handle_request(self, req, sock, addr):
+    def handle_request(self, listener, req, sock, addr):
         request_start = datetime.now()
+        environ = {}
+        resp = None
         try:
             self.cfg.pre_request(self, req)
-            resp, environ = wsgi.create(req, sock, addr, self.address, self.cfg)
+            resp, environ = wsgi.create(req, sock, addr,
+                    listener.getsockname(), self.cfg)
             self.nr += 1
             if self.alive and self.nr >= self.max_requests:
                 self.log.info("Autorestarting worker after current request.")
@@ -89,12 +101,19 @@ class AsyncWorker(base.Worker):
                 self.log.access(resp, req, environ, request_time)
             finally:
                 if hasattr(respiter, "close"):
-                  respiter.close()
+                    respiter.close()
             if resp.should_close():
                 raise StopIteration()
+        except Exception:
+            if resp.headers_sent:
+                # If the requests have already been sent, we should close the
+                # connection to indicate the error.
+                sock.shutdown(socket.SHUT_RDWR)
+                sock.close()
+            raise
         finally:
             try:
-                self.cfg.post_request(self, req, environ)
-            except:
-                pass
+                self.cfg.post_request(self, req, environ, resp)
+            except Exception:
+                self.log.exception("Exception in post_request hook")
         return True
